package im.zego.minigameenginedemo;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.os.Bundle;
import android.provider.Settings;
import android.text.TextUtils;
import android.util.Log;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.lifecycle.ViewModelProvider;

import com.gyf.barlibrary.BarHide;
import com.gyf.barlibrary.ImmersionBar;

import java.util.ArrayList;
import java.util.Locale;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;

import im.zego.minigameengine.ZegoGameInfo;
import im.zego.minigameengine.ZegoGameLanguage;
import im.zego.minigameengine.ZegoGameUserInfo;
import im.zego.minigameengine.ZegoMiniGameEngine;
import im.zego.minigameenginedemo.backend.BackendService;
import im.zego.minigameenginedemo.model.DemoGameInfo;
import im.zego.minigameenginedemo.permissions.api.ZegoPermission;
import im.zego.minigameenginedemo.ui.main.MiniGameMainFragment;
import im.zego.minigameenginedemo.ui.main.MiniGameMainViewModel;

public class MiniGameMainActivity extends AppCompatActivity {
    private static final String TAG = "MiniGameMainActivity";
    public static final String SP_FILENAME = "mini_game_sp";
    public static final String SP_LANGUAGE = "mini_language";
    public static final String SP_GAME_LANGUAGE = "mini_game_language";

    private static final String UPDATE_TOKEN_ALARM_ACTION = "im.zego.UPDATE_TOKEN_ALARM_ACTION";
    private ZegoMiniGameEngine miniGameEngine = ZegoMiniGameEngine.getInstance();
    private MiniGameMainViewModel viewModel;
    private Timer updateTokenTimer;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        ImmersionBar.with(this).hideBar(BarHide.FLAG_HIDE_BAR).init();
        getSupportActionBar().hide();
        // 启动前台服务
        Intent intent = new Intent(this, ForegroundService.class);
        startService(intent);
        // 初始化ViewModel
        viewModel = new ViewModelProvider(this).get(MiniGameMainViewModel.class);
        // 获取SharedPreferences实例
        SharedPreferences sp = getSharedPreferences(SP_FILENAME, Context.MODE_PRIVATE);
        // 读取语言设置
        viewModel.languageSetting = sp.getInt(SP_LANGUAGE, 0);
        viewModel.gameLanguageSetting = sp.getInt(SP_GAME_LANGUAGE, 0);
        // 根据语言设置更新应用语言
        if (viewModel.languageSetting != 0) {
            Resources res = getResources();
            Configuration config = res.getConfiguration();
            if (viewModel.languageSetting == 1) {
                config.locale = new Locale("zh");
            } else {
                config.locale = new Locale("en");
            }
            res.updateConfiguration(config, null);
        }

        // 设置布局文件
        setContentView(R.layout.activity_mini_game_main);
        // 检查并请求音视频权限
        if (!ZegoPermission.hasAudioPermission(getApplication()) || !ZegoPermission.hasCameraPermission(getApplication())) {
            ZegoPermission.requestCameraAndMicPermissionInner(this);
        }

        // 生成用户ID，名称和头像
        viewModel.userName = ZegoConstants.getRandomName();
        viewModel.userAvatar = ZegoConstants.getRandomAvatar();
        String androidID = Settings.System.getString(getContentResolver(), Settings.Secure.ANDROID_ID);
        if (TextUtils.isEmpty(androidID)) {
            androidID = UUID.randomUUID().toString();
        }
        // 用户ID根据业务方的实际情况来传，这里使用登录的手机号
        viewModel.userID = androidID;

        // 获取系统语言设置
        Locale locale = Locale.getDefault();
        String language = locale.getLanguage();
        if (language.equals("zh")) {
            viewModel.systemLanguage = ZegoGameLanguage.zh_CN;
        } else {
            for (ZegoGameLanguage lang : ZegoGameLanguage.values()) {
                // 模糊匹配
                if (language.equals(lang.toString())) {
                    viewModel.systemLanguage = lang;
                    break;
                }
            }
            // 其他都为英文
            if (viewModel.systemLanguage == null) {
                viewModel.systemLanguage = ZegoGameLanguage.en;
            }
        }

        // 如果是第一次启动，加载游戏主界面Fragment
        if (savedInstanceState == null) {
            getSupportFragmentManager().beginTransaction()
                    .replace(R.id.container, MiniGameMainFragment.newInstance(), "MiniGameMainFragment")
                    .commit();
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        // 检查定时器
        checkTimer();
        // 如果未初始化或全屏游戏列表为空，则进行初始化
        if (!miniGameEngine.isInit() || viewModel.fullScreenGameList.getValue() == null || viewModel.fullScreenGameList.getValue().size() == 0) {
            // 初始化游戏用户
            ZegoGameUserInfo userInfo = new ZegoGameUserInfo(viewModel.userID, viewModel.userName, viewModel.userAvatar);
            // 获取RTC Token
            BackendService.getToken(KeyCenter.appID, viewModel.userID, (errorCode, token) -> {
                Log.i(TAG, "[getRTCToken] errorCode:" + errorCode + ", token:" + token);
                if (errorCode != 0) {
                    return;
                }
                // 注册更新Token的广播接收器
                IntentFilter intentFilter = new IntentFilter(UPDATE_TOKEN_ALARM_ACTION);
                registerReceiver(updateTokenAlarmReceiver, intentFilter);
                // 设置定时器，每1个小时刷新一次Token
                setAlarm(60 * 60 * 1000);
                // 根据语言设置设置游戏语言
                if (viewModel.languageSetting == 0) {
                    miniGameEngine.setGameLanguage(viewModel.systemLanguage);
                } else if (viewModel.languageSetting == 1) {
                    miniGameEngine.setGameLanguage(ZegoGameLanguage.zh_CN);
                } else {
                    miniGameEngine.setGameLanguage(ZegoGameLanguage.en);
                }

                // 初始化MiniGameEngine
                miniGameEngine.init(KeyCenter.appID, token, userInfo, (errorCode2, object) -> {
                    Log.i(TAG, "miniGameEngine.init: " + errorCode2);

                    if (errorCode2 != 0) {
                        Toast.makeText(MiniGameMainActivity.this, getResources().getString(R.string.init_failed) + ZegoConstants.getErrorMsg(errorCode2), Toast.LENGTH_SHORT).show();
                    } else {
                        // 获取所有游戏列表
                        miniGameEngine.getAllGameList(
                                (errorCode3, gameList) -> {
                                    if (errorCode3 == 0) {
                                        if (gameList != null) {
                                            ArrayList<DemoGameInfo> allGameInfoList = new ArrayList<>();
                                            ArrayList<ZegoGameInfo> hostGameInfoList = new ArrayList<>();
                                            ArrayList<ZegoGameInfo> showGameInfoList = new ArrayList<>();
                                            ArrayList<ZegoGameInfo> fullScreenGameInfoList = new ArrayList<>();
                                            ArrayList<ZegoGameInfo> cloudGameInfoList = new ArrayList<>();
                                            ArrayList<ZegoGameInfo> inGameMatchGameInfoList = new ArrayList<>();
                                            for (ZegoGameInfo gameInfo : gameList) {
                                                DemoGameInfo demoGameInfo = new DemoGameInfo(gameInfo, ZegoConstants.EXCHANGE_POINT_NUMBER);
                                                allGameInfoList.add(demoGameInfo);

                                                if (gameInfo.gameMode.contains(1)) {
                                                    hostGameInfoList.add(gameInfo);
                                                }

                                                if (gameInfo.gameMode.contains(2)) {
                                                    showGameInfoList.add(gameInfo);
                                                }

                                                if (gameInfo.gameMode.contains(3)) {
                                                    fullScreenGameInfoList.add(gameInfo);
                                                }

                                                if (gameInfo.gameMode.contains(4)) {
                                                    cloudGameInfoList.add(gameInfo);
                                                }

                                                if (gameInfo.gameMode.contains(5)) {
                                                    inGameMatchGameInfoList.add(gameInfo);
                                                }
                                            }
                                            viewModel.allGameList.setValue(allGameInfoList);
                                            viewModel.hostGameList.postValue(hostGameInfoList);
                                            viewModel.showGameList.postValue(showGameInfoList);
                                            viewModel.fullScreenGameList.postValue(fullScreenGameInfoList);
                                            viewModel.cloudGameList.postValue(cloudGameInfoList);
                                            viewModel.inGameMatchGameList.postValue(inGameMatchGameInfoList);
                                        }
                                    } else {
                                        Toast.makeText(this, getResources().getString(R.string.get_game_list_failed) + ZegoConstants.getErrorMsg(errorCode3), Toast.LENGTH_SHORT).show();
                                    }
                                });
                    }
                });
            });
        }
    }

    // 设置定时器
    private void setAlarm(long triggerTime) {
        setLastTimer(triggerTime);
        if (updateTokenTimer != null) {
            updateTokenTimer.cancel();
        }
        updateTokenTimer = new Timer();
        updateTokenTimer.schedule(new TimerTask() {
            @Override
            public void run() {
                BackendService.getToken(KeyCenter.appID, viewModel.userID, (errorCode, token) -> {
                    if (errorCode == 0) {
                        // 更新token
                        miniGameEngine.updateToken(token);
                        setAlarm(60 * 60 * 1000);
                    } else {
                        setAlarm(10 * 1000);
                    }
                });
            }
        }, triggerTime);
    }

    // 更新Token的广播接收器
    private BroadcastReceiver updateTokenAlarmReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Log.i(TAG, "[updateTokenAlarmReceiver][onReceive]");
            BackendService.getToken(KeyCenter.appID, viewModel.userID, (errorCode, token) -> {
                if (errorCode == 0) {
                    // 更新token
                    miniGameEngine.updateToken(token);
                    setAlarm(60 * 60 * 1000);
                } else {
                    setAlarm(10 * 1000);
                }
            });
        }
    };

    @Override
    protected void onDestroy() {
        super.onDestroy();
        // 停止前台服务
        Intent intent = new Intent(this, ForegroundService.class);
        stopService(intent);
        // 反初始化MiniGameEngine
        miniGameEngine.unInit();
        // 注销广播接收器
        unregisterReceiver(updateTokenAlarmReceiver);
    }

    @Override
    public void onBackPressed() {
        if (!getSupportFragmentManager().popBackStackImmediate()) {
            finishAfterTransition();
        }
    }

    // 检查定时器
    private void checkTimer() {
        Log.i(TAG, "[checkTimer]");
        if (!miniGameEngine.isInit()) {
            return;
        }
        long lastTimer = getLastTimer();
        if (lastTimer < System.currentTimeMillis()) {
            // 上次alarm没有执行，这里需要手动执行
            Log.i(TAG, "[checkTimer][getToken]");
            BackendService.getToken(KeyCenter.appID, viewModel.userID, (errorCode, token) -> {
                if (errorCode == 0) {
                    // 更新token
                    miniGameEngine.updateToken(token);
                    setAlarm(60 * 60 * 1000);
                } else {
                    setAlarm(10 * 1000);
                }
            });
        }
    }

    // 获取上次定时器时间
    private long getLastTimer() {
        SharedPreferences sp = getSharedPreferences(SP_FILENAME, Context.MODE_PRIVATE);
        return sp.getLong("lastTimer", 0);
    }

    // 设置上次定时器时间
    private void setLastTimer(long time) {
        SharedPreferences sp = getSharedPreferences(SP_FILENAME, Context.MODE_PRIVATE);
        sp.edit().putLong("lastTimer", time).commit();
    }
}