package im.zego.minigameenginedemo;

import android.app.Application;
import android.text.TextUtils;
import android.view.TextureView;

import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

import im.zego.minigameenginedemo.rtcproxy.ExpressProxy;
import im.zego.zegoexpress.constants.ZegoStreamEvent;
import im.zego.zegoexpress.entity.ZegoRoomExtraInfo;

public class RTCEngineManager {
    private static final String TAG = "RTCEngineManager";
    public static final String STREAM_PREFIX = "zegotrans_";
    private static final String SUBTITLE_RECOGNIZING_TEXT_PREFIX = "zegoa2ti:";
    private static final String SUBTITLE_RECOGNIZED_TEXT_PREFIX = "zegoa2td:";
    private static RTCEngineManager instance = new RTCEngineManager();
    private ExpressProxy rtcEngine = new ExpressProxy(this);
    private String userID;
    private String userName;
    private String roomID;
    public String publishStreamID;
    private int memberCount = 0;
    private ArrayList<StreamInfo> streamInfoList = new ArrayList<>();
    private ArrayList<ZegoRoomExtraInfo> roomExtraInfoList = new ArrayList<>();
    private ArrayList<IRTCEventListener> rtcEventListenerList = new ArrayList<>();
    private boolean isInRoom = false;

    public static RTCEngineManager getInstance() {
        return instance;
    }

    private RTCEngineManager() {
    }

    public void initRtc(Application application) {
        rtcEngine.init(application);
    }

    public void initAudioGameConfig() {
        rtcEngine.initAudioGameConfig();
    }

    public void unInitRtc(IExecuteCallback<String> callback) {
        rtcEngine.unInit(callback);
        streamInfoList.clear();
    }

    public void loginRoom(String roomID, String roomName,
                          String userID, String userName, IExecuteCallback<Object> callback) {
        this.userID = userID;
        this.userName = userName;
        this.roomID = roomID;
        rtcEngine.login(roomID, roomName, userID, userName, callback);
        isInRoom = true;
    }

    public void logoutRoom() {
        publishStreamID = "";
        if (isInRoom) {
            roomExtraInfoList = null;
            streamInfoList.clear();
            isInRoom = false;
            rtcEngine.logout();
        }
    }

    private void startPublishStream(String streamID) {
        rtcEngine.startPublishStream(streamID);
    }

    public void stopPublishStream() {
        rtcEngine.stopPublishStream();
        for (StreamInfo streamInfo: streamInfoList) {
            if (streamInfo.streamID.equals(publishStreamID)) {
                streamInfoList.remove(streamInfo);
                break;
            }
        }
    }

    public void startPlayingStream(String streamID, Object view) {
        rtcEngine.startPlayingStream(streamID, view);
    }

    public void stopPlayingStream(String streamID) {
        rtcEngine.stopPlayingStream(streamID);
    }

    public boolean isInRoom() {
        return isInRoom;
    }

    public void startPublishStream(boolean isHost) {
        publishStreamID = STREAM_PREFIX + roomID + "_" + userID;
        if (isHost) {
            publishStreamID += "_host";
        }
        startPublishStream(publishStreamID);
        boolean find = false;
        for (StreamInfo streamInfo: streamInfoList) {
            if (streamInfo.streamID.equals(publishStreamID)) {
                find = true;
                break;
            }
        }

        if (!find) {
            StreamInfo streamInfo = new StreamInfo(userID, userName, publishStreamID);
            streamInfoList.add(0, streamInfo);
        }
    }

    public void startPreview(TextureView textureView) {
        rtcEngine.startPreview(textureView);
    }

    public ArrayList<StreamInfo> getStreamInfoList() {
        return streamInfoList;
    }

    public void setRoomExtraInfo(String roomID, String key, String value) {
        rtcEngine.setRoomExtraInfo(roomID, key, value);
    }

    public ArrayList<ZegoRoomExtraInfo> getRoomExtraInfoList() {
        return roomExtraInfoList;
    }

    public void muteMicrophone(boolean mute) {
        rtcEngine.muteMicrophone(mute);
    }

    public void muteSpeaker(String userID, boolean mute) {
        ArrayList<StreamInfo> streamInfoList = getStreamInfoList();
        for (StreamInfo streamInfo: streamInfoList) {
            if (streamInfo.userID.equals(userID)) {
                rtcEngine.muteSpeaker(streamInfo.streamID, mute);
            }
        }
    }

    public void onStreamAdd(ArrayList<StreamInfo> streamInfoList) {
        ArrayList<StreamInfo> newList = new ArrayList<>();
        for (StreamInfo streamInfo: streamInfoList) {
            boolean isFind = false;
            for (StreamInfo localStreamStreamInfo: this.streamInfoList) {
                if (streamInfo.streamID.equals(localStreamStreamInfo.streamID)) {
                    isFind = true;
                    break;
                }
            }

            if (isFind) {
                continue;
            } else {
                newList.add(streamInfo);
            }
        }
        if (!newList.isEmpty()) {
            this.streamInfoList.addAll(newList);
            for (IRTCEventListener listener : rtcEventListenerList) {
                listener.onStreamAdd(newList);
            }
        }
    }

    public void onStreamRemove(ArrayList<StreamInfo> streamInfoList) {
        for (StreamInfo streamInfo: streamInfoList) {
            for (StreamInfo sInfo: this.streamInfoList) {
                if (sInfo.streamID.equals(streamInfo.streamID)) {
                    this.streamInfoList.remove(sInfo);
                    break;
                }
            }
        }
        for (IRTCEventListener listener: rtcEventListenerList) {
            listener.onStreamRemove(streamInfoList);
        }
    }

    private void sendSEIData(String data) {
        rtcEngine.sendSEIData(data);
    }

    public void onReceiveSEI(String streamID, byte[] data) {
        String str = new String(data, StandardCharsets.UTF_8);
        if (str.startsWith(SUBTITLE_RECOGNIZING_TEXT_PREFIX)) {
            String subtitle = str.substring(SUBTITLE_RECOGNIZING_TEXT_PREFIX.length());
        } else if (str.startsWith(SUBTITLE_RECOGNIZED_TEXT_PREFIX)) {
            String subtitle = str.substring(SUBTITLE_RECOGNIZING_TEXT_PREFIX.length());
        }
    }

    public void onRoomExtraInfoUpdate(String roomID, ArrayList<ZegoRoomExtraInfo> arrayList) {
        roomExtraInfoList = arrayList;
        for (IRTCEventListener listener: rtcEventListenerList) {
            listener.onRoomExtraInfoUpdate(roomID, arrayList);
        }
    }

    public void loadSoundEffect(String localPath, IExecuteCallback<Integer> callback) {
        rtcEngine.loadSoundEffect(localPath, callback);
    }

    public void startSound(int soundEffectID, boolean isLoop, float volume) {
        rtcEngine.startSound(soundEffectID, isLoop, volume);
    }

    public void startSound(int soundEffectID, String path, boolean isLoop, float volume) {
        rtcEngine.startSound(soundEffectID, path, isLoop, volume);
    }

    public void stopSound(int soundEffectID) {
        rtcEngine.stopSound(soundEffectID);
    }

    public void changeSoundVolume(int soundEffectID, float volume) {
        rtcEngine.changeSoundVolume(soundEffectID, volume);
    }

    public void pauseSoundEffect() {
        rtcEngine.pauseSoundEffect();
    }

    public void resumeSoundEffect() {
        rtcEngine.resumeSoundEffect();
    }

    public void destroyZegoAudioEffectPlayer() {
        rtcEngine.destroyZegoAudioEffectPlayer();
    }

    public void onCaptureAudioData(ByteBuffer byteBuffer, int dataLength, int value) {
//        Log.i(TAG, "onCapturedAudioData:" + dataLength + ", " + value);
    }

    public void onRoomStateChanged(RoomStateChangeReason reason) {
        if (reason == RoomStateChangeReason.Reconnected) {
            if (!TextUtils.isEmpty(publishStreamID)) {
                startPublishStream(publishStreamID.endsWith("_host"));
            }
        } else {
            logoutRoom();
            for (IRTCEventListener listener : rtcEventListenerList) {
                listener.onRoomStateChanged(reason);
            }
        }
    }

    public void onRoomOnlineUserCountUpdate(String roomID, int count) {
        memberCount = count;
        for (IRTCEventListener listener : rtcEventListenerList) {
            listener.onRoomOnlineUserCountUpdate(roomID, count);
        }
    }

    public int getMemberCount() {
        return memberCount;
    }

    public void onPublisherStreamEvent(ZegoStreamEvent eventID, String streamID, String extraInfo) {
        for (IRTCEventListener listener : rtcEventListenerList) {
            listener.onPublisherStreamEvent(eventID, streamID, extraInfo);
        }
    }

    public interface IRTCEventListener {
        void onStreamAdd(ArrayList<StreamInfo> streamIDList);
        void onStreamRemove(ArrayList<StreamInfo> streamIDList);
        void onRoomStateChanged(RoomStateChangeReason reason);
        void onRoomExtraInfoUpdate(String roomID, ArrayList<ZegoRoomExtraInfo> arrayList);
        void onRoomOnlineUserCountUpdate(String roomID, int count);
        void onRoomUserUpdate(List<RTCUser> newUserList);
        void onPublisherStreamEvent(ZegoStreamEvent eventID, String streamID, String extraInfo);
    }

    public void addRTCEventListener(IRTCEventListener listener) {
        if (!rtcEventListenerList.contains(listener)) {
            rtcEventListenerList.add(listener);
        }
    }

    public void removeRTCEventListener(IRTCEventListener listener) {
        rtcEventListenerList.remove(listener);
    }

    public void onRoomUserUpdate(List<RTCUser> newUserList) {
        for (IRTCEventListener listener: rtcEventListenerList) {
            listener.onRoomUserUpdate(newUserList);
        }
    }

    public List<RTCUser> getRtcUserList() {
        return rtcEngine.getRtcUserList();
    }

    public static byte[] byteBufferToByteArray(ByteBuffer bytes) {
        int len = bytes.limit() - bytes.position();
        byte[] bytes1 = new byte[len];
        bytes.get(bytes1);
        return bytes1;
    }

    public enum RoomStateChangeReason {
        KickOut,
        Disconnect,
        LoginFailed,
        Reconnected
    }

    public static class StreamInfo {
        public String userID;
        public String userName;
        public String streamID;
        public StreamInfo(String userID, String userName, String streamID) {
            this.userID = userID;
            this.userName = userName;
            this.streamID = streamID;
        }
    }

    public static class RTCUser {
        public String userID;
        public String userName;
        public int type = 2; // 0 host， 1 self，2 other
        public RTCUser(String userID, String userName) {
            this.userID = userID;
            this.userName = userName;
        }
    }
}
